@@ -1,11 +1,12 @@
-use strict;
+use v5.20;
 use warnings;
-package Log::Dispatchouli;
+package Log::Dispatchouli 3.000; # TRIAL
 # ABSTRACT: a simple wrapper around Log::Dispatch
-$Log::Dispatchouli::VERSION = '2.020'; # TRIAL
+
 use Carp ();
 use File::Spec ();
 use Log::Dispatch;
+use Log::Fmt ();
 use Params::Util qw(_ARRAY0 _HASH0 _CODELIKE);
 use Scalar::Util qw(blessed weaken);
 use String::Flogger;
@@ -147,19 +148,11 @@ sub new {
         : ('stderr');
   };
 
-  my $pid_prefix = exists $arg->{log_pid} ? $arg->{log_pid} : 1;
-
-  my $self = bless {} => $class;
-
-  my $log = Log::Dispatch->new(
-    $pid_prefix
-    ? (
-        callbacks => sub {
-          "[$$] ". {@_}->{message}
-        },
-      )
-    : ()
-  );
+  my $log = Log::Dispatch->new;
+  my $self = bless {
+    dispatcher => $log,
+    log_pid    => (exists $arg->{log_pid} ? $arg->{log_pid} : 1),
+  } => $class;
 
   if ($arg->{to_file}) {
     require Log::Dispatch::File;
@@ -183,11 +176,19 @@ sub new {
         mode      => 'append',
         callbacks => do {
           if (my $format = $arg->{file_format}) {
-            sub { $format->({@_}->{message}) }
+            sub {
+              my $message = {@_}->{message};
+              $message = "[$$] $message" if $self->{log_pid};
+              $format->($message)
+            };
           } else {
             # The time format returned here is subject to change. -- rjbs,
             # 2008-11-21
-            sub { (localtime) . ' ' . {@_}->{message} . "\n" }
+            sub {
+              my $message = {@_}->{message};
+              $message = "[$$] $message" if $self->{log_pid};
+              (localtime) . " $message\n";
+            };
           }
         },
       )
@@ -195,20 +196,10 @@ sub new {
   }
 
   if ($arg->{facility} and not $self->env_value('NOSYSLOG')) {
-    require Log::Dispatch::Syslog;
-    $log->add(
-      Log::Dispatch::Syslog->new(
-        name      => 'syslog',
-        min_level => 'debug',
-        facility  => $arg->{facility},
-        ident     => $ident,
-        logopt    => 'pid',
-        socket    => $arg->{syslog_socket} || 'native',
-        callbacks => sub {
-          ( my $m = {@_}->{message} ) =~ s/\n/<LF>/g;
-          $m
-        },
-      ),
+    $self->setup_syslog_output(
+      facility  => $arg->{facility},
+      socket    => $arg->{syslog_socket},
+      ident     => $ident,
     );
   }
 
@@ -220,11 +211,12 @@ sub new {
         name      => 'self',
         min_level => 'debug',
         array     => $self->{events},
+        ($self->{log_pid} ? (callbacks => sub { "[$$] ". {@_}->{message} })
+                          : ())
       ),
     );
   }
 
-  $self->{dispatcher} = $log;
   $self->{prefix}     = $arg->{prefix};
   $self->{ident}      = $ident;
   $self->{config_id}  = $config_id;
@@ -251,12 +243,16 @@ for my $dest (qw(out err)) {
   my $name = "std$dest";
   my $code = sub {
     return if $_[0]->dispatcher->output($name);
+
+    my $callback = $_[0]->{log_pid} ? sub { "[$$] " . ({@_}->{message}) . "\n" }
+                                    : sub {           ({@_}->{message}) . "\n" };
+
     $_[0]->dispatcher->add(
       $_[0]->stdio_dispatcher_class->new(
         name      => "std$dest",
         min_level => 'debug',
         stderr    => ($dest eq 'err' ? 1 : 0),
-        callbacks => sub { +{@_}->{message} . "\n" },
+        callbacks => $callback,
         ($_[0]{quiet_fatal}{"std$dest"} ? (max_level => 'info') : ()),
       ),
     );
@@ -266,6 +262,26 @@ for my $dest (qw(out err)) {
   *{"enable_std$dest"} = $code;
 }
 
+sub setup_syslog_output {
+  my ($self, %arg) = @_;
+
+  require Log::Dispatch::Syslog;
+  $self->{dispatcher}->add(
+    Log::Dispatch::Syslog->new(
+      name      => 'syslog',
+      min_level => 'debug',
+      facility  => $arg{facility},
+      ident     => $arg{ident},
+      logopt    => ($self->{log_pid} ? 'pid' : ''),
+      socket    => $arg{socket} || 'native',
+      callbacks => sub {
+        ( my $m = {@_}->{message} ) =~ s/\n/<LF>/g;
+        $m
+      },
+    ),
+  );
+}
+
 #pod =method log
 #pod
 #pod   $logger->log(@messages);
@@ -296,11 +312,11 @@ sub log {
       my @flogged = map {; $flogger->flog($_) } @rest;
       $message    = @flogged > 1 ? $self->_join(\@flogged) : $flogged[0];
 
-      my $prefix  = _ARRAY0($arg->{prefix})
-                  ? [ @{ $arg->{prefix} } ]
-                  : [ $arg->{prefix} ];
+      my @prefix  = _ARRAY0($arg->{prefix})
+                  ? @{ $arg->{prefix} }
+                  : $arg->{prefix};
 
-      for (reverse grep { defined } $self->get_prefix, @$prefix) {
+      for (reverse grep { defined } $self->get_prefix, @prefix) {
         if (_CODELIKE( $_ )) {
           $message = $_->($message);
         } else {
@@ -368,6 +384,114 @@ sub log_debug {
   $self->log($arg, @rest);
 }
 
+#pod =method log_event
+#pod
+#pod This method is like C<log>, but is used for structured logging instead of free
+#pod form text.  It's invoked like this:
+#pod
+#pod   $logger->log($event_type => $data_ref);
+#pod
+#pod C<$event_type> should be a simple string, probably a valid identifier, that
+#pod identifies the kind of event being logged.  It is suggested, but not required,
+#pod that all events of the same type have the same kind of structured data in them.
+#pod
+#pod C<$data_ref> is a set of key/value pairs of data to log in this event.  It can
+#pod be an arrayref (in which case the ordering of pairs is preserved) or a hashref
+#pod (in which case they are sorted by key).
+#pod
+#pod The logged string will be in logfmt format, meaning a series of key=value
+#pod pairs separated by spaces and following these rules:
+#pod
+#pod =for :list
+#pod * an "identifier" is a string of printable ASCII characters between C<!> and
+#pod   C<~>, excluding C<\> and C<=>
+#pod * keys must be valid identifiers
+#pod * if a key is empty, C<~> is used instead
+#pod * if a key contains characters not permitted in an identifier, they are
+#pod   replaced by C<?>
+#pod * values must I<either> be valid identifiers, or be quoted
+#pod * quoted value start and end with C<">
+#pod * in a quoted value, C<"> becomes C<\">, C<\> becomes C<\\>, newline and
+#pod   carriage return become C<\n> and C<\r> respectively, and other control
+#pod   characters are replaced with C<\u{....}> where the contents of the braces are
+#pod   the hex value of the control character
+#pod
+#pod When values are undef, they are represented as C<~>.
+#pod
+#pod When values are array references, the index/values are mapped over, so that:
+#pod
+#pod   key => [ 'a', 'b' ]
+#pod
+#pod becomes
+#pod
+#pod   key.0=a key.1=b
+#pod
+#pod When values are hash references, the key/values are mapped, with keys sorted,
+#pod so that:
+#pod
+#pod   key => { b => 2, a => 1 }
+#pod
+#pod becomes
+#pod
+#pod   key.a=1 key.b=2
+#pod
+#pod This expansion is performed recursively.  If a value itself recurses,
+#pod appearances of a reference after the first time will be replaced with a string
+#pod like C<&foo.bar>, pointing to the first occurrence.  I<This is not meant to be
+#pod a robust serialization mechanism.>  It's just here to help you be a little
+#pod lazy.  Don't push the limits.
+#pod
+#pod If the value in C<$data_ref> is a code reference, it will be called and its
+#pod result logged.  If its result is also a code reference, you get whatever
+#pod garbage that code reference stringifies to.
+#pod
+#pod =cut
+
+sub log_event {
+  my ($self, $type, $data) = @_;
+
+  return $self->_log_event($type, undef, $data);
+}
+
+sub _compute_proxy_ctx_kvstr_aref {
+  return [];
+}
+
+sub _log_event {
+  my ($self, $type, $ctx, $data) = @_;
+
+  return if $self->get_muted;
+
+  my $kv_aref = Log::Fmt->_pairs_to_kvstr_aref([
+    event => $type,
+    (_ARRAY0($data) ? @$data : $data->%{ sort keys %$data })
+  ]);
+
+  splice @$kv_aref, 1, 0, @$ctx if $ctx;
+
+  $self->dispatcher->log(
+    level   => 'info',
+    message => join q{ }, @$kv_aref,
+  );
+
+  return;
+}
+
+#pod =method log_debug_event
+#pod
+#pod This method is just like C<log_event>, but will log nothing unless the logger
+#pod has its C<debug> property set to true.
+#pod
+#pod =cut
+
+sub log_debug_event {
+  my ($self, $type, $data) = @_;
+
+  return unless $self->get_debug;
+
+  $self->log_event($type, $data);
+}
+
 #pod =method set_debug
 #pod
 #pod   $logger->set_debug($bool);
@@ -601,6 +725,10 @@ sub clear_events {
 #pod = proxy_prefix
 #pod This is a prefix that will be applied to anything the proxy logger logs, and
 #pod cannot be changed.
+#pod = proxy_ctx
+#pod This is data to be inserted in front of event data logged through the proxy.
+#pod It will appear I<after> the C<event> key but before the logged event data.  It
+#pod can be in the same format as the C<$data_ref> argument to C<log_event>.
 #pod = debug
 #pod This can be set to true or false to change the proxy's "am I in debug mode?"
 #pod setting.  It can be changed or cleared later on the proxy.
@@ -615,12 +743,20 @@ sub proxy {
   my ($self, $arg) = @_;
   $arg ||= {};
 
-  $self->proxy_class->_new({
+  my $proxy = $self->proxy_class->_new({
     parent => $self,
     logger => $self,
     proxy_prefix => $arg->{proxy_prefix},
     (exists $arg->{debug} ? (debug => ($arg->{debug} ? 1 : 0)) : ()),
   });
+
+  if (my $ctx = $arg->{proxy_ctx}) {
+    $proxy->{proxy_ctx} = _ARRAY0($ctx)
+                        ? [ @$ctx ]
+                        : [ $ctx->%{ sort keys %$ctx } ];
+  }
+
+  return $proxy;
 }
 
 #pod =head2 parent
@@ -648,7 +784,7 @@ sub dispatcher   { $_[0]->{dispatcher} }
 #pod
 #pod This method is an experimental feature to allow you to pick an alternate
 #pod dispatch class for stderr and stdio.  By default, Log::Dispatch::Screen is
-#pod used.
+#pod used.  B<This feature may go away at any time.>
 #pod
 #pod =cut
 
@@ -724,7 +860,7 @@ Log::Dispatchouli - a simple wrapper around Log::Dispatch
 
 =head1 VERSION
 
-version 2.020
+version 3.000
 
 =head1 SYNOPSIS
 
@@ -760,6 +896,16 @@ logged as is, arrayrefs are taken as (sprintf format, args), and subroutines
 are called only if needed.  For more information read the L<String::Flogger>
 docs.
 
+=head1 PERL VERSION
+
+This library should run on perls released even a long time ago.  It should work
+on any version of perl released in the last five years.
+
+Although it may work on older versions of perl, no guarantee is made that the
+minimum required version will not be increased.  The version may be increased
+for any reason, and there is no promise that patches will be accepted to lower
+the minimum required perl.
+
 =head1 METHODS
 
 =head2 new
@@ -831,6 +977,90 @@ This method can also be called as C<debug>, to match other popular logging
 interfaces.  B<If you want to override this method, you must override
 C<log_debug> and not C<debug>>.
 
+=head2 log_event
+
+This method is like C<log>, but is used for structured logging instead of free
+form text.  It's invoked like this:
+
+  $logger->log($event_type => $data_ref);
+
+C<$event_type> should be a simple string, probably a valid identifier, that
+identifies the kind of event being logged.  It is suggested, but not required,
+that all events of the same type have the same kind of structured data in them.
+
+C<$data_ref> is a set of key/value pairs of data to log in this event.  It can
+be an arrayref (in which case the ordering of pairs is preserved) or a hashref
+(in which case they are sorted by key).
+
+The logged string will be in logfmt format, meaning a series of key=value
+pairs separated by spaces and following these rules:
+
+=over 4
+
+=item *
+
+an "identifier" is a string of printable ASCII characters between C<!> and C<~>, excluding C<\> and C<=>
+
+=item *
+
+keys must be valid identifiers
+
+=item *
+
+if a key is empty, C<~> is used instead
+
+=item *
+
+if a key contains characters not permitted in an identifier, they are replaced by C<?>
+
+=item *
+
+values must I<either> be valid identifiers, or be quoted
+
+=item *
+
+quoted value start and end with C<">
+
+=item *
+
+in a quoted value, C<"> becomes C<\">, C<\> becomes C<\\>, newline and carriage return become C<\n> and C<\r> respectively, and other control characters are replaced with C<\u{....}> where the contents of the braces are the hex value of the control character
+
+=back
+
+When values are undef, they are represented as C<~>.
+
+When values are array references, the index/values are mapped over, so that:
+
+  key => [ 'a', 'b' ]
+
+becomes
+
+  key.0=a key.1=b
+
+When values are hash references, the key/values are mapped, with keys sorted,
+so that:
+
+  key => { b => 2, a => 1 }
+
+becomes
+
+  key.a=1 key.b=2
+
+This expansion is performed recursively.  If a value itself recurses,
+appearances of a reference after the first time will be replaced with a string
+like C<&foo.bar>, pointing to the first occurrence.  I<This is not meant to be
+a robust serialization mechanism.>  It's just here to help you be a little
+lazy.  Don't push the limits.
+
+If the value in C<$data_ref> is a code reference, it will be called and its
+result logged.  If its result is also a code reference, you get whatever
+garbage that code reference stringifies to.
+
+=head2 log_debug_event
+
+This method is just like C<log_event>, but will log nothing unless the logger
+has its C<debug> property set to true.
+
 =head2 set_debug
 
   $logger->set_debug($bool);
@@ -903,7 +1133,7 @@ you're looking for.  Move along.
 
 This method is an experimental feature to allow you to pick an alternate
 dispatch class for stderr and stdio.  By default, Log::Dispatch::Screen is
-used.
+used.  B<This feature may go away at any time.>
 
 =head1 LOGGER PREFIX
 
@@ -1015,6 +1245,12 @@ C<%arg> is optional.  It may contain the following entries:
 This is a prefix that will be applied to anything the proxy logger logs, and
 cannot be changed.
 
+=item proxy_ctx
+
+This is data to be inserted in front of event data logged through the proxy.
+It will appear I<after> the C<event> key but before the logged event data.  It
+can be in the same format as the C<$data_ref> argument to C<log_event>.
+
 =item debug
 
 This can be set to true or false to change the proxy's "am I in debug mode?"
@@ -1075,11 +1311,11 @@ L<String::Flogger>
 
 =head1 AUTHOR
 
-Ricardo SIGNES <rjbs@cpan.org>
+Ricardo SIGNES <cpan@semiotic.systems>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Christopher J. Madsen Dagfinn Ilmari Mannsåker Dan Book George Hartzell Jon Stuart Matt Phillips Olivier Mengué Randy Stauner Sawyer X
+=for stopwords Christopher J. Madsen Dagfinn Ilmari Mannsåker Dan Book George Hartzell Jon Stuart Matt Phillips Olivier Mengué Randy Stauner Ricardo Signes Sawyer X
 
 =over 4
 
@@ -1117,13 +1353,21 @@ Randy Stauner <randy@magnificent-tears.com>
 
 =item *
 
+Ricardo Signes <rjbs@semiotic.systems>
+
+=item *
+
+Ricardo Signes <rjbs@users.noreply.github.com>
+
+=item *
+
 Sawyer X <xsawyerx@cpan.org>
 
 =back
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Ricardo SIGNES.
+This software is copyright (c) 2022 by Ricardo SIGNES.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
