@@ -1,8 +1,8 @@
 use strict;
 use warnings;
-package Log::Dispatchouli::Global;
+package Log::Dispatchouli::Global 3.000; # TRIAL
 # ABSTRACT: a system for sharing a global, dynamically-scoped logger
-$Log::Dispatchouli::Global::VERSION = '2.020'; # TRIAL
+
 use Carp ();
 use Log::Dispatchouli;
 use Scalar::Util ();
@@ -280,7 +280,7 @@ Log::Dispatchouli::Global - a system for sharing a global, dynamically-scoped lo
 
 =head1 VERSION
 
-version 2.020
+version 3.000
 
 =head1 DESCRIPTION
 
@@ -329,6 +329,16 @@ Log::Dispatchouli::Global is meant to be subclassed and shared only within
 controlled systems.  Remember, I<sharing your state with code you don't
 control is dangerous>.
 
+=head1 PERL VERSION
+
+This library should run on perls released even a long time ago.  It should work
+on any version of perl released in the last five years.
+
+Although it may work on older versions of perl, no guarantee is made that the
+minimum required version will not be increased.  The version may be increased
+for any reason, and there is no promise that patches will be accepted to lower
+the minimum required perl.
+
 =head1 USING
 
 In general, you will either be using a Log::Dispatchouli::Global class to get
@@ -436,11 +446,11 @@ else attempts to initialize the logger to something else.
 
 =head1 AUTHOR
 
-Ricardo SIGNES <rjbs@cpan.org>
+Ricardo SIGNES <cpan@semiotic.systems>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Ricardo SIGNES.
+This software is copyright (c) 2022 by Ricardo SIGNES.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
